﻿-- Work Order Tooltips
-- by Jadya - EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L

local sfind, tinsert = string.find, table.insert

local addon_enabled = false

local plots = {18,19,20,22,23,24,25,59,63}
-- alliance and horde garrison names
local garrison_names = {GetMapNameByID(971), GetMapNameByID(976)}

local function inGarrison()
 return tContains(garrison_names, GetZoneText())
end

local building_ids_for_names = {10,28,36,39,63,67,118,120,122,124,126,128,130,132,133,137,138,141,143,145,161,164}
local traders
local function generateTradersTable(tag)
 traders = { 
 -- large
 [L.Get(tag.."bunker")]         = {8,9,10},
 -- medium
 [L.Get(tag.."tradingpost")]    = {111,144,145},
 [L.Get(tag.."barn")]           = {24,25,133},
 [L.Get(tag.."lumbermill")]     = {40,41,138},
 [L.Get(tag.."gladiator")]      = {159,160,161},
 -- small
 [L.Get(tag.."leatherworking")] = {90,121,122},
 [L.Get(tag.."inscription")]    = {95,129,130},
 [L.Get(tag.."enchanting")]     = {93,125,126},
 [L.Get(tag.."alchemy")]        = {76,119,120},
 [L.Get(tag.."engineering")]    = {91,123,124},
 [L.Get(tag.."blacksmithing")]  = {60,117,118},
 [L.Get(tag.."jewelcrafting")]  = {96,131,132},
 [L.Get(tag.."tailoring")]      = {94,127,128},
 [L.Get(tag.."garden")]         = {29,136,137},
 [L.Get(tag.."mine")]           = {61,62,63},
}

 for _,bID in pairs(building_ids_for_names) do
  local _, bName = C_Garrison.GetBuildingInfo(bID)
  table.insert(garrison_names, bName)
 end
end

WOTSaves = {}
local hookedframes = {}
local tooltips = {}
local pool = {}
local function getframe()
 local f = pool[1]
 if f then
  table.remove(pool, 1) 
 else
  f = CreateFrame("Frame")
 end
 
 return f
end

local frames_backdrop  = { bgFile = "Interface\\Buttons\\GreyscaleRamp64", 
                           edgeFile = "Interface\\Buttons\\WHITE8X8", 
						   edgeSize = 1
						   }

local lastupdate = 0
local function updateTooltip(tt, elapsed, forced)
 if not addon_enabled then
  tt:Hide()
 end
 
 lastupdate = lastupdate + elapsed
 if lastupdate > 0.33 or forced then lastupdate = 0 else return end

 if tt.buildingID == 0 then
  tt:Clear()
  tt.title:SetText(QUEUE_TIME_UNAVAILABLE)
 end

 local bName, bIcon, maxWO, completedWO, currentWO, _, WOduration, currWOtimeleft, WIname, WIicon, _, WIID = C_Garrison.GetLandingPageShipmentInfo(tt.buildingID)
 
 local loot_string = WIname and ("- |T"..WIicon..":12|t "..WIname.." - ") or ""
 local queued_string = currentWO and currentWO - completedWO or 0
 queued_string = queued_string.." / ".. (maxWO and maxWO or 0)

 tt.title:SetText("|cffcccc00"..bName.."|r")
 tt.leftfont:SetText(READY..": \n"
                   ..BATTLEFIELD_QUEUE_STATUS..": \n"
                   ..NEXT..": " .."\n\n"
                   ..loot_string)
 tt.rightfont:SetText("|cff00FF00"..(completedWO or 0).."\n"
                    ..queued_string.."\n"
                    ..(currWOtimeleft or NOT_APPLICABLE).."|r")
 local w = math.max(tt.rightfont:GetWidth(), tt.leftfont:GetWidth()) + 10
 if tt:GetWidth() < w then tt:SetWidth(w) end
end

local function buildTooltip(f, name)

 local bID = 0
 local follower_data = ""
 
 for _,v in pairs(traders[name]) do
  for _,plotID in pairs(plots) do
   local buildingID, buildingName, texturePrefix, icon, description, rank, currencyID, currencyAmount, goldAmount, timeRequirement, needsPlan, isPreBuilt, possSpecs, upgrades, canUpgrade, isMaxLevel, hasFollowerSlot, knownSpecs, currentSpec, specCooldown, isBeingBuilt, timeStarted, buildDuration, timeRemainingText, canCompleteBuild = C_Garrison.GetOwnedBuildingInfo(plotID)
   if buildingID == v then
    bID = v
    if (hasFollowerSlot) then
     local followerName = C_Garrison.GetFollowerInfoForBuilding(plotID)
     if (followerName) then
      follower_data = format(GARRISON_BUILDING_FOLLOWER_WORKING, "|cff00FF00"..followerName.."|r")
     else
      follower_data = "|cffFF000"..GARRISON_BUILDING_FOLLOWER_EMPTY.."|r"
     end
     break
	end
   end
  end
  if bID > 0 then break end
 end
 
 if bID == 0 then
  return
 end

 local tt = getframe()
 tooltips[name] = tt

 if not tt.initialized then
  tt:SetSize(150,120)
  tt:SetPoint("BOTTOM", f, "TOP")
  tt:SetBackdrop(frames_backdrop)
  tt:SetBackdropColor(0.07,0.07,0.07,0.9)
  tt:SetBackdropBorderColor(0.5,0.5,0.1,0.9)

  tt.title = tt:CreateFontString()
  tt.title:SetFont("Fonts\\MORPHEUS.ttf", 16, "")
  tt.leftfont = tt:CreateFontString(nil, "OVERLAY", "GameTooltipText")
  tt.rightfont = tt:CreateFontString(nil, "OVERLAY", "GameTooltipText")
  tt.additionaldata = tt:CreateFontString(nil, "OVERLAY", "GameTooltipText")
  tt.title:SetPoint("TOP",0, -5)
  tt.rightfont:SetJustifyH("RIGHT")
  tt.leftfont:SetPoint("TOPLEFT",5,-25)
  tt.rightfont:SetPoint("TOPRIGHT",-5,-25)
  tt.additionaldata:SetPoint("BOTTOM",0, 5)
  tt:SetScript("OnHide", function(self)
   for k,v in pairs(tooltips) do
    if v == self then
	 table.insert(pool, v)
	 tooltips[k] = nil
	 v:Hide()
	 break
	end
   end
  end)
  
  tt.Clear = function()
   tt.title:SetText("")
   tt.rightfont:SetText("")
   tt.leftfont:SetText("")
   tt.additionaldata:SetText("")
  end
  
  tt.initialized = true
 end

 tt:SetParent(f)
 tt.buildingID = bID
 tt:Clear()

 tt.additionaldata:SetText(follower_data)
 
 tt:SetWidth(math.max(150, tt.additionaldata:GetWidth() + 10))
 tt:SetScript("OnUpdate", function(self, elapsed) updateTooltip(self, elapsed, name) end)
 tt:SetParent(f)
 tt:SetPoint("BOTTOM", f, "TOP")
 updateTooltip(tt, 0, true)
 tt:Show()
end

local function isFriendly(r,g,b)
 return (r < 0.1 and g > 0.1 and b < 0.1) or
        (r < 0.1 and g < 0.1 and b > 0.1)
end

local function nameplateOnShow(self)

 if not addon_enabled then return end

 local name = self:GetName()
 if name and sfind(name, "NamePlate") then
  local hb, cb = self:GetChildren()
  local r,g,b = hb:GetChildren():GetStatusBarColor()
  
  if not isFriendly(r,g,b) then return end
  
  local unitname = cb:GetRegions():GetText()
  
  if traders[unitname] then
   buildTooltip(self, unitname)
  else
   self:Hide()
  end
 end
end

local function checkNameplates(m, ...)
 local maxN = m or WorldFrame:GetNumChildren()
 for i = 1, maxN do
  local f = select(i, ...) or select(i, WorldFrame:GetChildren())
  local name = f:GetName()
  if  not tContains(hookedframes, f) and name and name:find('^NamePlate%d') then
   f:HookScript("OnShow", nameplateOnShow)
   nameplateOnShow(f)
   tinsert(hookedframes, f)
  end
 end
end

local function setNameplateOption(v)
 if not WOTSaves.changeplates then
  SetCVar("nameplateShowFriends", v)
 end
end

local interval = 0
local maxNameplates = 0
local f = CreateFrame("Frame")
local lastT = 0
f:SetScript("OnUpdate", function(self, elapsed)

 interval = interval + elapsed
 if interval < 0.33 then return end
 interval = 0
 
 if not inGarrison() then
  if addon_enabled then
   setNameplateOption(0)
   addon_enabled = false
  end
  return
 end
 
 if not addon_enabled then
  setNameplateOption(1)
  addon_enabled = true
 end
 
 local maxN = WorldFrame:GetNumChildren()
 if maxN ~= maxNameplates then
  checkNameplates(maxN, WorldFrame:GetChildren())
  maxNameplates = maxN
 end
end)

local eventframe = CreateFrame("Frame")
eventframe:RegisterEvent("PLAYER_ENTERING_WORLD")
eventframe:RegisterEvent("VARIABLES_LOADED")
eventframe:RegisterEvent("SHIPMENT_CRAFTER_CLOSED")
eventframe:RegisterEvent("CHAT_MSG_LOOT")
eventframe:RegisterEvent("CHAT_MSG_CURRENCY")
eventframe:RegisterEvent("PLAYER_REGEN_DISABLED")
eventframe:RegisterEvent("PLAYER_REGEN_ENABLED")
local function eventhandler(self, event, ...)
 if event == "PLAYER_ENTERING_WORLD" then
  faction = UnitFactionGroup("player")
  local tag = faction == "Alliance" and "A_" or "H_"
  generateTradersTable(tag)
  eventframe:UnregisterEvent("PLAYER_ENTERING_WORLD")
 elseif event == "VARIABLES_LOADED" then
  addonTable:SetupConfigFrame()
 elseif addon_enabled and (event == "SHIPMENT_CRAFTER_CLOSED" or event == "CHAT_MSG_LOOT" or event == "CHAT_MSG_CURRENCY") and lastT < time() - 1 then
  C_Garrison.RequestLandingPageShipmentInfo()
  lastT = time()
 elseif event == "PLAYER_REGEN_DISABLED" then
  if inGarrison() then
   setNameplateOption(0)
  end
 elseif event == "PLAYER_REGEN_ENABLED" then
  if inGarrison() then
   setNameplateOption(1)
  end
 end
end
eventframe:SetScript("OnEvent", eventhandler)